#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>
#include <stdbool.h>
#include <stdlib.h>

#include "ApplicationServices/ApplicationServices.h"
#include "MacVirtualKeys.h"

void keyDown(unsigned int pKeyCode) {
    CGEventRef eventRef;
    eventRef = CGEventCreateKeyboardEvent (NULL, (CGKeyCode)pKeyCode, true);
    CGEventPost(kCGSessionEventTap, eventRef);
    CFRelease(eventRef);
}
 
void keyUp(unsigned int pKeyCode) {
    CGEventRef eventRef;
    eventRef = CGEventCreateKeyboardEvent (NULL, (CGKeyCode)pKeyCode, false);
    CGEventPost(kCGSessionEventTap, eventRef);
    CFRelease(eventRef);
}

#define SERVER_PORT 4242

bool keystates[22] = {false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false};
char buttons_n_h[22] = {'A', 'B', 'X', 'Y', 'L', 'R', 'U', 'D', 'L', 'R', 'U', 'D', 'L', 'R', 'U', 'D', 'P', 'M', 'Q', 'W', 'E', 'T'};
unsigned int SCval[22] = {VK_A, VK_B, VK_X, VK_Y, VK_L, VK_R, VK_U, VK_D, VK_L, VK_R, VK_U, VK_D, VK_L, VK_R, VK_U, VK_D, VK_P, VK_M, VK_Q, VK_W, VK_E, VK_T};
const char * real_names[] = {"A", "B", "X", "Y", "LEFT", "RIGHT", "UP", "DOWN", "LEFT", "RIGHT", "UP", "DOWN", "LEFT", "RIGHT", "UP", "DOWN", "+", "-", "ZL", "ZR", "L", "R"};

int main()
{
	printf("== RemotePad client ==\n");
	printf("Starting client...\n");

	int sock;
	struct sockaddr_in name;
	struct hostent *hp, *gethostbyname();
	int bytes;
	
	// Create socket from which to read
	sock = socket(AF_INET, SOCK_DGRAM, 0);	
	if (sock < 0)
		return 1;
  
	// Bind our local address so that the client can send to us */
	memset((char*)&name, 0, sizeof(name));
	name.sin_family      = AF_INET;
	name.sin_addr.s_addr = htonl(INADDR_ANY);
	name.sin_port        = htons(SERVER_PORT);
  
	if (bind(sock, (struct sockaddr *) &name, sizeof(name)))
		return 2;
	
	if (ntohs(name.sin_port) != 4242)
		return 3;
	
	printf("Ready for connections!\n");
	
	// Allocate buffer
	char * incoming = malloc(1024);

	while ((bytes = read(sock, incoming, 1024)) > 0) 
	{
		for (int i = 0; i < 22; i++) 
		{
			if (keystates[i] == false && incoming[i] == buttons_n_h[i]) 
			{
				keyDown(SCval[i]);
				keystates[i] = true;
				printf("Key Press:   %c (%s)\n", buttons_n_h[i], real_names[i]);
			} 
			else if (keystates[i] == true && incoming[i] != buttons_n_h[i]) 
			{
				keyUp(SCval[i]);
				keystates[i] = false;
				printf("Key Release: %c (%s)\n", buttons_n_h[i], real_names[i]);
			}
		}
	}
	
	free(incoming);
	
	close(sock);
	return 0;
}
